from component.rpc.messagebus.caller import MessagebusCaller
from component.rpc.messagebus.server import MessagebusServer
from component.rpc.rpc import Caller, Server

assert Caller
assert Server
assert MessagebusCaller
assert MessagebusServer
