from zrb.builtin import (
    base64,
    devtool,
    docker,
    env,
    eval,
    explain,
    generator,
    git,
    md5,
    process,
    project,
    say,
    schedule,
    ubuntu,
    update,
    version,
    watch_changes,
)

assert base64
assert devtool
assert docker
assert env
assert eval
assert explain
assert generator
assert git
assert md5
assert process
assert project
assert ubuntu
assert update
assert say
assert schedule
assert version
assert watch_changes
